
% Evans function for a Sturm-Liouville problem

clear all

TIME = clock;                    % length of time for the simulation

global lambda2

%------------------------User input parameters-------------------------

N = 96;                          % number of data points

bi1 = 0.0;  
bi2 = -1.0;                      % boundary coefficients at left bound of interval
bf1 = 0.0; 
bf2 = -1.0;                      % boundary coefficients at right bound of interval

lambda0 = -500.0;                % eigenvalue find start on real axis
lambdaf = 0.0001;                % eigenvalue find finish on real axis
lambday0 = -1;                   % eigenvalue find start on imaginary axis
lambdayf = 1;                    % eigenvalue find finish on imaginary axis
step = 1;                        % Domain coloring grid step size

intervalStart = 0;               % Left bound of spatial interval
intervalEnd = 1;                 % Right bound of spatial interval

%---Variables for CIF evaluation---
n = 64;                          % Number of divisions for trapezoidal rule w/large contour
nPrime = 4;                      % Number of divisions for trapezoidal rule w/small derivative contour
R = 3.0;                         % Radius of large contour
r = 0.1;                         % Radius of small contour
wnR = 30;                        % Radius for winding number calculation

%------------------------------------------------------------------------

b10 = bi1; b20 = bi2; b11 = bf1; b21 = bf2;

%--------------------Finite Difference Approximation---------------------

h = 1/(N);                                 % step size

if b20 == 0 && b21 ==0                     % Here, p(x) must be 1
    x = intervalStart+h:h:intervalEnd-h;   % Discretize for Dirichlet
elseif b20 == 0                     
    x = intervalStart+h:h:intervalEnd;
elseif b21 == 0                     
    x = intervalStart:h:intervalEnd-h;     
else
    x = intervalStart:h:intervalEnd;       % Discretize for non-Dirichlet
end

syms temp                                  % Temporary variable for coefficient function
u = [1,1,1,1]; lambda2 = 1;                % Does not matter at this point
[udot, Pt, Qt, Wt] = EvansFunctionVF(temp,u);
Q = diag(eval(subs(Qt,temp,x)));           % q(x) function diagonal matrix
W = diag(eval(subs(Wt,temp,x)));           % w(x) function diagonal matrix

% the boundary conditions for the ODE system
px0 = eval(subs(Pt,temp,intervalStart));
px1 = eval(subs(Pt,temp,intervalEnd));
u0 = [-b20*px0 ; b10 ; 0 ; 0]; 
u1 = [-b21*px1 ; b11];


if b20 == 0 && b21 ==0  
     
    % Create DPD matrix
    shift = h/2;
    pCenter = -eval(subs(Pt,temp,x+shift)) - eval(subs(Pt,temp,x-shift));
    xR = intervalStart+h:h:intervalEnd-2*h;   
    pRight = eval(subs(Pt,temp,xR+shift));
    xL = intervalStart+2*h:h:intervalEnd-h;
    pLeft = eval(subs(Pt,temp,xL-shift));
    DPDsize = length(pCenter)-1;

    p0shift = eval(subs(Pt,temp,intervalStart + shift));
    pNshift = eval(subs(Pt,temp,intervalEnd - shift));

    syms xp
    pprime = diff(Pt,temp);


    DPD = (1/(h^2)) * (diag(pCenter) + diag(pRight,+1) + diag(pLeft,-1));
    
elseif b20 == 0  
      
    % Create DPD matrix
    shift = h/2;
    pCenter = -eval(subs(Pt,temp,x+shift)) - eval(subs(Pt,temp,x-shift));
    xR = intervalStart+h:h:intervalEnd-h;
    pRight = eval(subs(Pt,temp,xR+shift));
    xL = intervalStart+2*h:h:intervalEnd;
    pLeft = eval(subs(Pt,temp,xL-shift));
    DPDsize = length(pCenter)-1;

    p0shift = eval(subs(Pt,temp,intervalStart + shift));
    pNshift = eval(subs(Pt,temp,intervalEnd - shift));

    syms xp
    pprime = diff(Pt,temp);


    DPD = (1/(h^2)) * (diag(pCenter) + diag(pRight,+1) + diag(pLeft,-1));
   
    DPD(DPDsize+1,DPDsize+1) = 2*(1/h^2)*((-h*b11/(b21))*px1 -...
        (-h*h*b11/(2*b21))*subs(pprime,temp,intervalEnd) -...
        pNshift);                                                % Last entry in D11 matrix defined by B.C.

    DPD(DPDsize+1,DPDsize) = 2*pNshift/(h^2);                    % Second to last entry in D11 matrix not part of tri-diag matrix

    
elseif b21 ==0  
      
    % Create DPD matrix
    shift = h/2;
    pCenter = -eval(subs(Pt,temp,x+shift)) - eval(subs(Pt,temp,x-shift));
    xR = intervalStart:h:intervalEnd-2*h;   
    pRight = eval(subs(Pt,temp,xR+shift));
    xL = intervalStart+h:h:intervalEnd-h;
    pLeft = eval(subs(Pt,temp,xL-shift));
    DPDsize = length(pCenter)-1;

    p0shift = eval(subs(Pt,temp,intervalStart + shift));
    pNshift = eval(subs(Pt,temp,intervalEnd - shift));

    syms xp
    pprime = diff(Pt,temp);

    DPD = (1/(h^2)) * (diag(pCenter) + diag(pRight,+1) + diag(pLeft,-1));
        
    DPD(1,1) = (2*(1/h)^2) *(-p0shift +...
        (b10/b20)*h*px0 -...
        ((h^2)/2)*(b10/b20)*subs(pprime,temp,intervalStart));    % First entry in D11 matrix defined by B.C.

    DPD(1,2) = 2*(1/h^2)*p0shift;                                % Second entry in D11 matrix not part of tri-diag matrix

    
else
    
    % Create DPD matrix
    shift = h/2;
    pCenter = -eval(subs(Pt,temp,x+shift)) - eval(subs(Pt,temp,x-shift));
    xR = intervalStart:h:intervalEnd-h;   
    pRight = eval(subs(Pt,temp,xR+shift));
    xL = intervalStart+h:h:intervalEnd;
    pLeft = eval(subs(Pt,temp,xL-shift));
    DPDsize = length(pCenter)-1;

    p0shift = eval(subs(Pt,temp,intervalStart + shift));
    pNshift = eval(subs(Pt,temp,intervalEnd - shift));

    syms xp
    pprime = diff(Pt,temp);


    DPD = (1/(h^2)) * (diag(pCenter) + diag(pRight,+1) + diag(pLeft,-1));
    
    
    DPD(1,1) = (2*(1/h)^2) *(-p0shift +...
        (b10/b20)*h*px0 -...
        ((h^2)/2)*(b10/b20)*subs(pprime,temp,intervalStart));    % First entry in D11 matrix defined by B.C.

    DPD(1,2) = 2*(1/h^2)*p0shift;                                % Second entry in D11 matrix not part of tri-diag matrix

    DPD(DPDsize+1,DPDsize+1) = 2*(1/h^2)*((-h*b11/(b21))*px1 -...
        (-h*h*b11/(2*b21))*subs(pprime,temp,intervalEnd) -...
        pNshift);                                                % Last entry in D11 matrix defined by B.C.

    DPD(DPDsize+1,DPDsize) = 2*pNshift/(h^2);                    % Second to last entry in D11 matrix not part of tri-diag matrix
end
 
[TEST, evalApprox] = eig(DPD-Q,W);                               % Find finite difference eigenvalues


% Limit to evals within user-defined region
counter = 0;
for j = 1:length(x)
    if evalApprox(j,j) ~= 0 && real(evalApprox(j,j)) > lambda0 && real(evalApprox(j,j)) < lambdaf && imag(evalApprox(j,j)) > lambday0 && imag(evalApprox(j,j)) < lambdayf
        counter = counter + 1;
        boundedRegiont(counter) = evalApprox(j,j);      
    end
end
boundedRegion = boundedRegiont.';

%---------------Zeros of the Evans Function using the CIF-----------------

%---Parameters for ODE solver---
evansplot = [intervalStart ; (intervalStart+intervalEnd)/2 ; intervalEnd];
options = odeset('AbsTol',1e-8,'RelTol',1e-6);

% ---------------CIF zero approximation--------------

evalcounter = 0;
centers = boundedRegion;    % Use finite difference approximations as initial estimate for center of contour

for j = 1:length(centers)
        
        % Create a vector of points on the contour
        X = 0:2*pi/n:2*pi;
        pointsLarge = centers(j) + R.*cos(X) + 1i.*R.*sin(X);
        
        % Create and fill vectors of the Evans Function and derivative estimates
        evaluatedDenom = zeros(1,n+1);
        FPrimes = zeros(1,n+1);
        
        for num = 1:n+1
            lambda2 = pointsLarge(num);
            [Evans,EvansPrime] = EvansEvaluation(evansplot,u0,u1,options);
            evaluatedDenom(num) = Evans;
            FPrimes(num) = EvansPrime;
        end
        
        % Create and fill a vector to approx zeros with the CIF
        
        rvector = R*cos(X) + 1i*R*sin(X);
        Evans_est = pointsLarge .* (FPrimes ./ evaluatedDenom) .* rvector;
        zero = (1/(2*pi))*trapz(X,Evans_est);
        
        % Make vector to hold zeros
        evalcounter = evalcounter + 1;
        Evanszerot(evalcounter) = zero;
        
end  
Evanszero = real(Evanszerot.'); 

% ----------------Plot Eigenfunction--------------------------------------


numpts = 64; 
evansplot = linspace(intervalStart,intervalEnd, numpts);

if length(Evanszero) < 5;
    figure
    for num = 1:length(Evanszero)
        lambda2 = Evanszero(num);
        [taxis, yaxis] = ode45(@EvansFunctionVF,evansplot,u0,options);
        aa = trapz(taxis, yaxis(1:numpts,1).^2);
        fff = yaxis(1:numpts,1) / aa.^0.5;
        subplot(2,2,num);
        plot(taxis,fff); %yaxis(1:numpts, 4));
        title(['\lambda = ', num2str(lambda2,'%8.3f')])
    end
elseif length(Evanszero) < 7;
    figure
    for num = 1:length(Evanszero)
        lambda2 = Evanszero(num);
        [taxis, yaxis] = ode45(@EvansFunctionVF,evansplot,u0,options);
        aa = trapz(taxis, yaxis(1:numpts,1).^2);
        fff = yaxis(1:numpts,1) / aa.^0.5;
        subplot(2,3,num);
        plot(taxis,fff); %yaxis(1:numpts, 4));
        title(['\lambda = ', num2str(lambda2,'%8.3f')])
    end
elseif length(Evanszero) < 9;
    figure
    for num = 1:length(Evanszero)
        lambda2 = Evanszero(num);
        [taxis, yaxis] = ode45(@EvansFunctionVF,evansplot,u0,options);
        aa = trapz(taxis, yaxis(1:numpts,1).^2);
        fff = yaxis(1:numpts,1) / aa.^0.5;
        subplot(2,4,num);
        plot(taxis,fff); %yaxis(1:numpts, 4));
        title(['\lambda = ', num2str(lambda2,'%8.3f')])
    end
elseif length(Evanszero) < 13;
    figure
    for num = 1:length(Evanszero)
        lambda2 = Evanszero(num);
        [taxis, yaxis] = ode45(@EvansFunctionVF,evansplot,u0,options);
        aa = trapz(taxis, yaxis(1:numpts,1).^2);
        fff = yaxis(1:numpts,1) / aa.^0.5;
        subplot(3,4,num);
        plot(taxis,fff); %yaxis(1:numpts, 4));
        title(['\lambda = ', num2str(lambda2,'%8.3f')])
    end
elseif length(Evanszero) > 13;
    figure
    for num = 1:length(Evanszero)
        lambda2 = Evanszero(num);
        [taxis, yaxis] = ode45(@EvansFunctionVF,evansplot,u0,options);
        aa = trapz(taxis, yaxis(1:numpts,1).^2);
        fff = yaxis(1:numpts,1) / aa.^0.5;
        subplot(4,4,num);
        plot(taxis,fff); %yaxis(1:numpts, 4));
        title(['\lambda = ', num2str(lambda2,'%8.3f')])
    end
end    

TOTtime = etime(clock,TIME);
disp(['Computational time = ', num2str(TOTtime,'%8.3f'), ' seconds'])

